
package com.tmerclub.cloud.multishop.mapper;

import com.tmerclub.cloud.multishop.dto.ShopUserDTO;
import com.tmerclub.cloud.multishop.model.ShopUser;
import com.tmerclub.cloud.multishop.vo.ShopUserVO;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * @author tmerclub
 * @date 2020/09/03
 */
public interface ShopUserMapper {

    /**
     * 获取店铺用户列表
     *
     * @param shopUserDTO 搜索参数
     * @return 店铺用户列表
     */
    List<ShopUserVO> listByShopId(@Param("shopUser") ShopUserDTO shopUserDTO);

    /**
     * 根据用户id获取商家用户信息
     *
     * @param userId 用户id
     * @return 商家用户信息
     */
    ShopUserVO getByUserId(@Param("userId") Long userId);

    /**
     * 保存商家用户信息
     *
     * @param shopUser
     */
    void save(@Param("shopUser") ShopUser shopUser);

    /**
     * 更新店铺用户信息
     *
     * @param shopUser
     */
    void update(@Param("shopUser") ShopUser shopUser);

    /**
     * 根据店铺用户id删除店铺用户
     *
     * @param shopUserId
     */
    void deleteById(@Param("shopUserId") Long shopUserId);

    /**
     * 获取店主账号的用户id-第一个创建的账号（仅用于审核店铺）
     *
     * @param shopId
     * @return
     */
    Long getUserIdByShopId(@Param("shopId") Long shopId);

    /**
     * 根据用户id更新店铺id
     *
     * @param shopId
     * @param shopUserId
     * @return
     */
    int updateShopIdByShopUserId(@Param("shopId") Long shopId, @Param("shopUserId") Long shopUserId);

    /**
     * 获取指定的用户的信息列表
     *
     * @param userIds
     * @return
     */
    List<ShopUser> listShopUserByUserIds(@Param("userIds") List<Long> userIds);

    /**
     * 根据手机号码查用户数量
     * @param mobile 手机号码
     * @return 数量
     */
    int countAccoutByMobile(@Param("mobile") String mobile);
}
