
package com.tmerclub.cloud.api.group.feign;

import com.tmerclub.cloud.api.group.vo.GroupActivityVO;
import com.tmerclub.cloud.common.product.vo.GroupActivitySpuVO;
import com.tmerclub.cloud.common.response.ServerResponseEntity;

import java.util.Date;
import java.util.List;

/**
 * @author YXF
 * @date 2021/3/29
 */
public interface GroupFeignClient {

    /**
     * 获取活动开始的时间（预热的活动开始时间为当前）
     * @param activityId 活动id
     * @return void
     */
    ServerResponseEntity<Date> getActivityStartTime(Long activityId);

    /**
     * 获取活动信息
     * @param activityId 活动id
     * @return void
     */
    ServerResponseEntity<GroupActivityVO> getActivityInfo(Long activityId);

    /**
     * 根据商品id获取团购商品信息
     * @param spuIds 商品id
     * @return void
     */
    ServerResponseEntity<List<GroupActivitySpuVO>> groupSpuListBySpuIds(List<Long> spuIds);

    /**
     * 根据商品ids下线所有的团购活动
     * @param spuIds 商品ids
     * @return 返回结果
     */
    ServerResponseEntity<Void> offlineGroupBySpuIds(List<Long> spuIds);

    /**
     * 根据skuIds更新团购活动
     * @param skuIds
     * @param spuId
     * @return
     */
    ServerResponseEntity<Void> updateGroupActivityPrice(List<Long> skuIds, Long spuId);

    /**
     * 根据商品id和状态修改拼团活动状态
     * @param spuId
     * @return
     */
    ServerResponseEntity<Void> updateGroupActivityStatus(Long spuId);
}
